unit LCatTool;

{$X+} {Extended syntax}
{$V-} {No string var length checking}

interface
Uses
  DefLib,
  Forms, Dialogs, Classes, SysUtils, Controls, Graphics, WinProcs,
  IniFiles;

const
  C_ProgDescr   = 'Listing catalog';
  C_ProgIdn     = 'LCAT';
  C_ProgVers    = 'v1.9';
  C_ProgTarget  = {$ifdef Win32} '(32 bit)' {$else} '(16 bit)' {$endif};
  C_Copyright   = 'Copyright  1997-2000, J.R. Ferguson';
  C_AuthorEmail = 'j.r.ferguson@iname.com';
  C_AuthorURL   = 'http://hello.to/ferguson';
  C_IniFileExt  = '.INI';
  C_DosFileExt  = '.lca';
  C_LngFileExt  = '.lce';
  C_GrpFileExt  = '.lcg';

  HC_General    = 20;
  HC_MainData   = 30;
  HC_GroupData  = 40;

  C_FileFilter  =
    'DOS name cat (*'+C_DosFileExt+')|*'+C_DosFileExt+'|'+
    'Long name cat (*'+C_LngFileExt+')|*'+C_LngFileExt+'|'+
    'All files (*.*)|*.*';

       {Normal order         Group order}
  C_FnmCol1Dos  = 01; C_FnmCol2Dos  = 05; C_FnmLenDos = 08;
  C_ExtCol1Dos  = 10; C_ExtCol2Dos  = 14; C_ExtLenDos = 03;
  C_GrpCol1Dos  = 14; C_GrpCol2Dos  = 01; C_GrpLenDos = 03;

  C_FnmCol1Lng  = 01; C_FnmCol2Lng  = 05; C_FnmLenLng = 29;
  C_ExtCol1Lng  = 31; C_ExtCol2Lng  = 35; C_ExtLenLng = 03;
  C_GrpCol1Lng  = 35; C_GrpCol2Lng  = 01; C_GrpLenLng = 03;

  C_GrpIdnCol   = 01; C_GrpIdnLen = 03;
  C_GrpDscCol   = 05; C_GrpDscLen = MaxStp;

  C_HeaderFnmDos= 'Filename Ext Grp Group description';
  C_HeaderGrpDos= 'Grp Filename Ext Group description';
                  {....v....1....v....2....v....3....v....4....v....5....v}
  C_HeaderFnmLng= 'Filename                      Ext Grp Group description';
  C_HeaderGrpLng= 'Grp Filename                      Ext Group description';

type
  T_LastFind    = (lf_Unknown, lf_Fnm, lf_Grp);
  T_CatType     = (ct_DosNames,ct_LongNames);

  T_Options     = record
    DataFile    : String;
    OptCatType  : T_CatType;
    OptGrpOrder,
    OptToolbar,
    OptHeader,
    OptStatusbar,
    OptHints,
    OptSavePos  : boolean;
    WndTop,
    WndLeft,
    WndHeight,
    WndWidth    : integer;
  end;

const
  C_DflOptions  : T_Options =
  ( DataFile    : '' ;
    OptCatType  : {$ifdef Win32} ct_LongNames {$else} ct_DosNames {$endif};
    OptGrpOrder : false;
    OptToolbar  : true;
    OptHeader   : true;
    OptStatusbar: true;
    OptHints    : true;
    OptSavePos  : true;
    WndTop      : 0;
    WndLeft     : 0;
    WndHeight   : 500;
    WndWidth    : 400);

var
  GV_ProgIdn    : String[8];
  GV_SettingFnm : String;
  GV_HeaderFnm  : String;
  GV_HeaderGrp  : String;
  GV_FnmCol1, GV_FnmCol2, GV_FnmCol, GV_FnmLen : StpInd;
  GV_ExtCol1, GV_ExtCol2, GV_ExtCol, GV_ExtLen : StpInd;
  GV_GrpCol1, GV_GrpCol2, GV_GrpCol, GV_GrpLen : StpInd;

function  ErrMsg(V_Condition: boolean; V_Msg: String): boolean;
function  ToOrdGrp(const V_Item: StpTyp): StpTyp;
function  ToOrdFnm(const V_Item: StpTyp): StpTyp;
function  ToCatType(V_CatType: T_CatType; const V_Item: StpTyp): StpTyp;
procedure SetCatType(V_CatType: T_CatType);
procedure SetGroupOrder(V_GroupOrder: boolean);
function  FullGrpStr(const V_Grp: StpTyp; V_GrpList: TStringList): StpTyp;
function  SetListItem(const V_Fnm, V_Ext, V_Grp: StpTyp): StpTyp;
function  GetListFnm(const V_Item: StpTyp): StpTyp;
function  GetListExt(const V_Item: StpTyp): StpTyp;
function  GetListFnmExt(const V_Item: StpTyp): StpTyp;
function  GetListGrp(const V_Item: StpTyp): StpTyp;
function  SetGrpListItem(const V_Ident, V_Descr: StpTyp): StpTyp;
function  GetGrpListIdent(const V_Item: StpTyp): StpTyp;
function  GetGrpListDescr(const V_Item: StpTyp): StpTyp;
function  ReadOptions(var V_Options: T_Options): boolean;
procedure WriteOptions(var V_Options: T_Options);


implementation
Uses
  LcatMain, LCatEGrp,
  StfLib, StpLib, NumLib;

function ErrMsg(V_Condition: boolean; V_Msg: String): boolean;
begin
  if V_Condition then
    MessageDlg(GV_ProgIdn+': '+V_Msg,mtError,[mbOK],0);
  ErrMsg:= V_Condition;
end;

function  ToOrdGrp(const V_Item: StpTyp): StpTyp;
begin
  ToOrdGrp:= StfSub(V_Item,GV_GrpCol1,GV_GrpLen)+' '+
             StfSub(V_Item,GV_FnmCol1,GV_FnmLen+1+GV_ExtLen);
end;

function  ToOrdFnm(const V_Item: StpTyp): StpTyp;
begin
  ToOrdFnm:= StfSub(V_Item,GV_FnmCol2,GV_FnmLen+1+GV_ExtLen)+' '+
             StfSub(V_Item,GV_GrpCol2,GV_GrpLen);
end;

function  ToCatType(V_CatType: T_CatType; const V_Item: StpTyp): StpTyp;
begin case V_CatType of
  ct_DosNames  : ToCatType:=
    StfUpp(StfSub(V_Item,C_FnmCol1Lng,C_FnmLenDos))+' '+
    StfUpp(StfSub(V_Item,C_ExtCol1Lng,C_ExtLenDos))+' '+
    StfSub(V_Item,C_GrpCol1Lng,C_GrpLenLng);
  ct_LongNames : ToCatType:=
    StfFill(StfSub(V_Item,C_FnmCol1Dos,C_FnmLenDos),' ',C_FnmLenLng+1)+
    StfFill(StfSub(V_Item,C_ExtCol1Dos,C_ExtLenDos),' ',C_ExtLenLng+1)+
    StfFill(StfSub(V_Item,C_GrpCol1Dos,C_GrpLenDos),' ',C_GrpLenLng);
end; end;

procedure SetCatType(V_CatType: T_CatType);
begin
  case V_CatType of
    ct_DosNames : begin
      GV_FnmCol1:= C_FnmCol1Dos; GV_FnmCol2:= C_FnmCol2Dos; GV_FnmLen:= C_FnmLenDos;
      GV_ExtCol1:= C_ExtCol1Dos; GV_ExtCol2:= C_ExtCol2Dos; GV_ExtLen:= C_ExtLenDos;
      GV_GrpCol1:= C_GrpCol1Dos; GV_GrpCol2:= C_GrpCol2Dos; GV_GrpLen:= C_GrpLenDos;
      GV_HeaderFnm:= C_HeaderFnmDos; GV_HeaderGrp:= C_HeaderGrpDos;
      with MainWnd do begin
        DlgFileOpen.DefaultExt := StfAfter(C_DosFileExt,'.');
        DlgFileSave.DefaultExt := StfAfter(C_DosFileExt,'.');
        DlgFileOpen.FilterIndex:= 1;
        DlgFileSave.FilterIndex:= 1;
      end;
    end;
    ct_LongNames : begin
      GV_FnmCol1:= C_FnmCol1Lng; GV_FnmCol2:= C_FnmCol2Lng; GV_FnmLen:= C_FnmLenLng;
      GV_ExtCol1:= C_ExtCol1Lng; GV_ExtCol2:= C_ExtCol2Lng; GV_ExtLen:= C_ExtLenLng;
      GV_GrpCol1:= C_GrpCol1Lng; GV_GrpCol2:= C_GrpCol2Lng; GV_GrpLen:= C_GrpLenLng;
      GV_HeaderFnm:= C_HeaderFnmLng; GV_HeaderGrp:= C_HeaderGrpLng;
      with MainWnd do begin
        DlgFileOpen.DefaultExt := StfAfter(C_LngFileExt,'.');
        DlgFileSave.DefaultExt := StfAfter(C_LngFileExt,'.');
        DlgFileOpen.FilterIndex:= 2;
        DlgFileSave.FilterIndex:= 2;
      end;
    end;
  end;
  MainWnd.Options.OptCatType:= V_CatType;
  SetGroupOrder(MainWnd.Options.OptGrpOrder);
end;

procedure SetGroupOrder(V_GroupOrder: boolean);
begin
  MainWnd.Options.OptGrpOrder := V_GroupOrder;
  if V_GroupOrder then begin
    GV_FnmCol:= GV_FnmCol2;
    GV_ExtCol:= GV_ExtCol2;
    GV_GrpCol:= GV_GrpCol2;
    MainWnd.Header.Caption:= GV_HeaderGrp;
  end
  else begin
    GV_FnmCol:= GV_FnmCol1;
    GV_ExtCol:= GV_ExtCol1;
    GV_GrpCol:= GV_GrpCol1;
    MainWnd.Header.Caption:= GV_HeaderFnm;
  end;
end;

function FullGrpStr(const V_Grp: StpTyp; V_GrpList: TStringList): StpTyp;
var i: integer; FoundGrp: boolean;
begin with V_GrpList do begin
  if Count = 0 then FullGrpStr:= V_Grp
  else begin
    i:= -1; FoundGrp:= false;
    while (not FoundGrp) and (i < Count - 1) do begin
      Inc(i);
      FoundGrp:= GetGrpListIdent(V_GrpList[i]) = V_Grp;
    end;
   if FoundGrp then FullGrpStr:= V_GrpList[i] else FullGrpStr:= V_Grp;
  end;
end; end;

function  SetListItem(const V_Fnm, V_Ext, V_Grp: StpTyp): StpTyp;
begin
  if MainWnd.Options.OptGrpOrder then SetListItem:=
    StfFill(StfTrunc(V_Grp,GV_GrpLen),' ',GV_GrpLen+1)+
    StfFill(StfTrunc(V_Fnm,GV_FnmLen),' ',GV_FnmLen+1)+
    StfFill(StfTrunc(V_Ext,GV_ExtLen),' ',GV_ExtLen)
  else SetListItem:=
    StfFill(StfTrunc(V_Fnm,GV_FnmLen),' ',GV_FnmLen+1)+
    StfFill(StfTrunc(V_Ext,GV_ExtLen),' ',GV_ExtLen+1)+
    StfFill(StfTrunc(V_Grp,GV_GrpLen),' ',GV_GrpLen);
end;

function  GetListFnm(const V_Item: StpTyp): StpTyp;
begin GetListFnm:= StfRTS(StfSub(V_Item,GV_FnmCol,GV_FnmLen)); end;

function  GetListExt(const V_Item: StpTyp): StpTyp;
begin GetListExt:= StfRTS(StfSub(V_Item,GV_ExtCol,GV_ExtLen)); end;

function  GetListFnmExt(const V_Item: StpTyp): StpTyp;
begin
  GetListFnmExt:= StfRTS(StfSub(V_Item,GV_FnmCol,GV_FnmLen+1+GV_ExtLen));
end;

function  GetListGrp(const V_Item: StpTyp): StpTyp;
begin GetListGrp:= StfRTS(StfSub(V_Item,GV_GrpCol,GV_GrpLen)); end;

function SetGrpListItem(const V_Ident, V_Descr: StpTyp): StpTyp;
begin
  SetGrpListItem:=
    StfFill(StfTrunc(V_Ident,C_GrpIdnLen),' ',C_GrpIdnLen+1) + V_Descr;
end;

function GetGrpListIdent(const V_Item: StpTyp): StpTyp;
begin
  GetGrpListIdent:= StfRTS(StfSub(V_Item,C_GrpIdnCol,C_GrpIdnLen));
end;

function GetGrpListDescr(const V_Item: StpTyp): StpTyp;
begin
  GetGrpListDescr:= StfRTS(StfSub(V_Item,C_GrpDscCol,C_GrpDscLen));
end;


function ReadOptions(var V_Options: T_Options): boolean;
var IniFile: TIniFile; ok: boolean;

  procedure GetFont(const V_Section: String; V_Font: TFont);
  var bold, italic: boolean;
  begin with IniFile, V_Font do begin
    Name   := ReadString (V_Section, 'Name'  ,Name  );
    Size   := ReadInteger(V_Section, 'Size'  ,Size  );
    Color  := ReadInteger(V_Section, 'Color' ,Size  );
    bold   := ReadBool   (V_Section, 'Bold'  ,fsBold   in Style);
    italic := ReadBool   (V_Section, 'Italic',fsItalic in Style);
    Style:= [];
    if bold   then Style:= Style + [fsBold];
    if italic then Style:= Style + [fsItalic];
  end end;

begin with V_Options do begin {ReadOptions}
  ok:= false;
  Screen.Cursor:= crHourGlass;
  IniFile:= TIniFile.Create(GV_SettingFnm);
  try
    with IniFile do begin
      DataFile    := ReadString ('Data'    ,'FileName'  ,DataFile     );
      OptCatType  :=
           T_CatType(ReadInteger('Data'    ,'CatType'   ,ord(ct_DosNames)));
      OptGrpOrder := ReadBool   ('Options' ,'ByGroup'   ,OptGrpOrder  );
      OptToolbar  := ReadBool   ('Options' ,'Toolbar'   ,OptToolbar   );
      OptHeader   := ReadBool   ('Options' ,'Header'    ,OptHeader    );
      OptStatusBar:= ReadBool   ('Options' ,'Statusbar' ,OptStatusBar );
      OptHints    := ReadBool   ('Options' ,'Hints'     ,OptHints     );
      OptSavePos  := ReadBool   ('Options' ,'SavePos'   ,OptSavePos   );
      WndTop      := ReadInteger('Position','Top'       ,WndTop       );
      WndLeft     := ReadInteger('Position','Left'      ,WndLeft      );
      WndHeight   := ReadInteger('Position','Height'    ,WndHeight    );
      WndWidth    := ReadInteger('Position','Width'     ,WndWidth     );
    end;
    with MainWnd do begin
      GetFont('ListFont'   ,ListBox.Font);
      GetFont('GrpListFont',GrpLstFont);
      Header.Font:= ListBox.Font;
    end;
    ok:= true;
  finally
    IniFile.Free;
    Screen.Cursor:= crDefault;
    ReadOptions:= ok;
  end;
end; end;


procedure WriteOptions(var V_Options: T_Options);
var IniFile: TIniFile;

  procedure PutFont(const V_Section: String; V_Font: TFont);
  begin with IniFile, V_Font do begin
    WriteString (V_Section,'Name'  ,Name             );
    WriteInteger(V_Section,'Size'  ,Size             );
    WriteInteger(V_Section,'Color' ,Color            );
    WriteBool   (V_Section,'Bold'  ,fsBold   in Style);
    WriteBool   (V_Section,'Italic',fsItalic in Style);
  end end;

begin with V_Options do begin {WriteOptions}
  IniFile:= TIniFile.Create(GV_SettingFnm);
  Screen.Cursor:= crHourGlass;
  try try
    if OptSavePos then with MainWnd do begin
      WndTop    := Top;
      WndLeft   := Left;
      WndHeight := Height;
      WndWidth  := Width;
    end;
    with IniFile do begin
      WriteString ('Data'       ,'FileName'  ,DataFile     );
      WriteInteger('Data'       ,'CatType'   ,ord(OptCatType));
      WriteBool   ('Options'    ,'ByGroup'   ,OptGrpOrder  );
      WriteBool   ('Options'    ,'Header'    ,OptHeader    );
      WriteBool   ('Options'    ,'Toolbar'   ,OptToolbar   );
      WriteBool   ('Options'    ,'Statusbar' ,OptStatusBar );
      WriteBool   ('Options'    ,'Hints'     ,OptHints     );
      WriteBool   ('Options'    ,'SavePos'   ,OptSavePos   );
      WriteInteger('Position'   ,'Top'       ,WndTop       );
      WriteInteger('Position'   ,'Left'      ,WndLeft      );
      WriteInteger('Position'   ,'Height'    ,WndHeight    );
      WriteInteger('Position'   ,'Width'     ,WndWidth     );
      PutFont     ('ListFont'   ,MainWnd.ListBox.Font);
      PutFont     ('GrpListFont',DlgGrpList.GrpList.Font);
    end;
  finally Screen.Cursor:= crDefault; IniFile.Free; end;
  except on E: Exception do MessageDlg(E.Message,mtError,[mbOK],0) end;
end; end;

end.
